#XXXXXXXXXXXXXXXXXXXXXXXXXXX#
# --- Support functions --- #
#XXXXXXXXXXXXXXXXXXXXXXXXXXX#

# Convert an ltraj to a list of SpatialLines ---------------------------------
# INPUT:
# ltraj from adehabitatLT-library
# OUTPUT:
# list of SpatialLines from sp-library
ltraj2SL <- function(traj){
  require(sp)
  toto <- function(x){
    sl <- SpatialLines(list(Lines(apply(cbind(x[-nrow(x),c(1,2)], x[-1,c(1,2)], attributes(x)$id), 1, 
                                        function(x){Line(rbind(as.numeric(x[c(1,2)]),as.numeric(x[c(3,4)])))}), ID=x[1,5])))  
    return(sl)
  }
  
  for (i in c(1:length(traj))){z <- toto(traj[[i]])}
  
  lsl <- lapply(traj, toto)
  return(lsl)
}


# Sample random steps matched to the observed steps from a trajectory ---------------------------------
# INPUT:
# x: ltraj from adehabitatLT-library
# n_control: number of controls for each step
# sl_max: the maximum step length for the random steps
# OUTPUT:
# list with case-control steps
casecontrol_steps <- function(traj, n_control=2, sl_max=15){
  if (!inherits(traj, "ltraj")) 
    stop("requires an ltraj-object for input")
  
  toto <- function(x){
    y <- data.frame(dt=rep(x$dt[-nrow(x)], each=n_control+1), x1=rep(x$x[-nrow(x)], each=n_control+1), y1=rep(x$y[-nrow(x)], each=n_control+1),  
                    x2=rep(x$x[-1], each=n_control+1), y2=rep(x$y[-1], each=n_control+1),
                    step_id = rep(c(2:nrow(x))-1, each=n_control+1), case=rep(c(1, rep(0, n_control)), nrow(x)-1),
                    sl = runif((nrow(x)-1)*(n_control+1), 0, sl_max), aa=runif((nrow(x)-1)*(n_control+1), 0, 2*pi))
    y$aa[y$case==1] <- atan2(y$y2[y$case==1]-y$y1[y$case==1], y$x2[y$case==1]-y$x1[y$case==1])
    y$sl[y$case==1] <- sqrt((y$x2[y$case==1]-y$x1[y$case==1])^2+(y$y2[y$case==1]-y$y1[y$case==1])^2)
    
    y$x2[y$case==0] <- y$x1[y$case==0]+y$sl[y$case==0]*cos(y$aa[y$case==0])
    y$y2[y$case==0] <- y$y1[y$case==0]+y$sl[y$case==0]*sin(y$aa[y$case==0])
    y$id <- c(1:nrow(y))
    y$animal_id <- attr(x, "id")
    
    y <- y[,c("id", "animal_id", "step_id", "dt", "case", "x1", "y1", "x2", "y2", "sl", "aa")]
    return(y)
  }
  y <- lapply(traj, toto)
  class(y) <- c("ccsteps", "list")
  attr(y, "env_annotated") <- FALSE
  return(y)
}

# Extract the values from a raster brick along steps from a case-control step object ---------------------------------
# INPUT:
# x: ccsteps, generated by casecontrol_steps-function
# rb: raster brick
# OUTPUT:
# list with case-control steps and environmental attributes
annotate_ccsteps <- function(steps, rb){
  if (!inherits(steps, "ccsteps")) 
    stop("requires a case-control steps (ccsteps) object for input")
  if (attr(steps, "env_annotated")) 
    stop("steps were already annotated")
  require(raster)
  
  toto <- function(x){
    sl <- apply(x[,c("x1","y1","x2","y2")], 1, function(x){Line(rbind(x[c(1,2)],x[c(3,4)]))})
    sl <- lapply(c(1:length(sl)), function(x, sl, id){Lines(sl[[x]], ID=id[x])}, sl=sl, id=x$id)
    sl <- SpatialLines(sl)  
    y <- extract(rb, sl, fun=mean)
    y <- cbind(x,y)
  }
  
  steps <- lapply(steps, toto)
    
  class(steps) <- c("ccsteps", "list")
  attr(steps, "env_annotated") <- TRUE
  return(steps)
}


